/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.schedule;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.zayac.admin.agent.model.entity.FinanceDO;
import com.zayac.admin.agent.model.entity.StatsDO;
import com.zayac.admin.agent.model.req.FinanceSumReq;
import com.zayac.admin.agent.service.FinanceService;
import com.zayac.admin.agent.service.FinanceSumService;
import com.zayac.admin.agent.service.StatsService;
import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.constant.ApiPathConstants;
import com.zayac.admin.req.AgentDataVisualListReq;
import com.zayac.admin.req.PayRecordListReq;
import com.zayac.admin.req.team.TeamFinanceReq;
import com.zayac.admin.req.team.TeamInfoReq;
import com.zayac.admin.req.team.TeamMemberReq;
import com.zayac.admin.resp.*;
import com.zayac.admin.resp.team.*;
import com.zayac.admin.service.*;
import com.zayac.admin.system.model.entity.RoleDO;
import com.zayac.admin.system.model.entity.UserDO;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.*;
import com.zayac.admin.utils.TableFormatter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import top.continew.starter.core.exception.BusinessException;

import java.math.BigDecimal;
import java.text.NumberFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.YearMonth;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;

@Component
@RequiredArgsConstructor
@Slf4j
public class DailyReport {
    private final UserService userService;
    private final TeamService teamService;
    private final RoleService roleService;
    private final UserRoleService userRoleService;
    private final AccountService accountService;
    private final TelegramMessageService telegramMessageService;
    private final AgentDataVisualListService agentDataVisualListService;
    private final StatsService statsService;
    private final CompletableFutureFinanceService completableFutureFinanceService;
    private final FinanceService financeService;
    private final FinanceSumService financeSumService;
    private final CompletableFutureWebClientService completableFutureWebClientService;
    private final Executor asyncTaskExecutor;

    private static final String MINISTER_ROLE_CODE = "minister";

    @Scheduled(cron = "0 40 11,14,17,21 * * ?")
    public void teamAccountDailyReport() {
        LocalDateTime nowDateTime = LocalDateTime.now();
        LocalDate nowDate = LocalDate.now();
        sendDailyReport(nowDate, nowDate.atStartOfDay(), nowDateTime);
    }

    @Scheduled(cron = "0 15 0 * * ?")
    public void dailySummarize() {
        LocalDate yesterday = LocalDate.now().minusDays(1);
        sendDailyReport(yesterday, yesterday.atStartOfDay(), LocalDateTime.of(yesterday, LocalTime.MAX));
        getPayFailedMember(yesterday);
        saveData(yesterday);
    }

    /**
     * 查询存款失败用户,并发送消息
     *
     * @param date 日期
     */
    private void getPayFailedMember(LocalDate date) {
        RoleDO minister = roleService.getByCode(MINISTER_ROLE_CODE);
        List<Long> userIds = userRoleService.listUserIdByRoleId(minister.getId());
        TeamMemberReq memberListReq = TeamMemberReq.builder()
                .registerStartDate(date)
                .registerEndDate(date)
                .startDate(date)
                .endDate(date)
                .registerSort(1)
                .status(1)
                .pageSize(100)
                .build();

        userIds.forEach(userId -> {
            List<CompletableFuture<List<TeamMember>>> futureList = new ArrayList<>();
            UserDO ministerUser = userService.getById(userId);
            List<AccountResp> accounts = accountService.getAccountsByUserId(userId, DisEnableStatusEnum.ENABLE)
                    .stream()
                    .filter(AccountResp::getIsTeam)
                    .toList();

            accounts.forEach(account -> {
                CompletableFuture<MemberPagination<List<TeamMember>>> memberPaginationCompletableFuture = completableFutureWebClientService
                        .fetchDataForAccount(account, ApiPathConstants.MEMBER_TEAM_LIST_URL, memberListReq, new ParameterizedTypeReference<>() {
                        });

                CompletableFuture<List<TeamMember>> teamMembersFuture = memberPaginationCompletableFuture.thenApply(MemberPagination::getList)
                        .thenApplyAsync(members -> members.stream()
                                .filter(member -> member.getDeposit() != null && member.getDeposit().compareTo(BigDecimal.ZERO) == 0)
                                .collect(Collectors.toList()), asyncTaskExecutor)
                        .thenComposeAsync(membersWithoutDep -> {
                            List<CompletableFuture<TeamMember>> memberFutures = membersWithoutDep.stream()
                                    .map(memberWithoutDep -> {
                                        PayRecordListReq req = PayRecordListReq.builder()
                                                .startDate(date)
                                                .endDate(date)
                                                .pageSize(100)
                                                .id(memberWithoutDep.getId())
                                                .build();
                                        CompletableFuture<PayRecordList<List<PayRecord>>> completableFuture = completableFutureWebClientService
                                                .fetchDataForAccount(account, ApiPathConstants.PAY_RECORD_LIST_URL, req, new ParameterizedTypeReference<>() {
                                                });
                                        return completableFuture.thenApplyAsync(pagination -> {
                                            if (pagination.getOrderAmountTotal().compareTo(BigDecimal.ZERO) > 0
                                                    && pagination.getScoreAmountTotal().compareTo(BigDecimal.ZERO) == 0) {
                                                return memberWithoutDep;
                                            } else {
                                                return null;
                                            }
                                        }, asyncTaskExecutor);
                                    }).toList();

                            return CompletableFuture.allOf(memberFutures.toArray(new CompletableFuture[0]))
                                    .thenApply(v -> memberFutures.stream()
                                            .map(CompletableFuture::join)
                                            .filter(Objects::nonNull)
                                            .collect(Collectors.toList()));
                        }, asyncTaskExecutor)
                        .thenApplyAsync(membersWithoutDep -> {
                            // 发送给每个account关联的user用户
                            if (!membersWithoutDep.isEmpty()) {
                                Map<String, List<TeamMember>> groupByTopAgentName = membersWithoutDep.stream()
                                        .collect(Collectors.groupingBy(TeamMember::getTopAgentName));
                                groupByTopAgentName.forEach((accountName, accountMembers) -> {
                                    String notification = telegramMessageService
                                            .buildFailedPayMessage(accountName, accountMembers);
                                    UserDO currUser = accountService.getUserByAccountUsername(accountName);
                                    if (currUser != null && DisEnableStatusEnum.ENABLE.equals(currUser.getNeedNotify())) {
                                        String botToken = StrUtil.isEmpty(currUser.getBotToken()) ? ministerUser.getBotToken() : currUser.getBotToken();
                                        telegramMessageService.sendMessage(botToken, currUser.getRegAndDepIds(), notification);
                                    }
                                });
                            }
                            return membersWithoutDep;
                        }, asyncTaskExecutor);

                futureList.add(teamMembersFuture);
            });

            CompletableFuture<Void> allFutures = CompletableFuture.allOf(futureList.toArray(new CompletableFuture[0]));
            allFutures.thenRunAsync(() -> {
                // 主线下的所有的存款失败用户
                List<TeamMember> allTeamMembers = futureList.stream()
                        .map(CompletableFuture::join)
                        .flatMap(List::stream)
                        .toList();
                Map<String, List<TeamMember>> groupByTopAgentName = new TreeMap<>(allTeamMembers.stream()
                        .collect(Collectors.groupingBy(TeamMember::getTopAgentName)));
                StringBuilder combinedNotification = new StringBuilder();
                groupByTopAgentName.forEach((accountName, accountMembers) -> {
                    String notification = telegramMessageService
                            .buildFailedPayMessage(accountName, accountMembers);
                    combinedNotification.append(notification).append("\n");
                });
                telegramMessageService
                        .sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, combinedNotification.toString());
                if (ministerUser != null && DisEnableStatusEnum.ENABLE.equals(ministerUser.getNeedNotify())) {
                    telegramMessageService.sendMessage(ministerUser.getBotToken(), ministerUser.getReportIds(), combinedNotification.toString());
                }

            }, asyncTaskExecutor).exceptionally(ex -> {
                log.error("Error collecting and processing data", ex);
                return null;
            });
        });
    }


    private void sendDailyReport(LocalDate reportDate, LocalDateTime startDateTime, LocalDateTime endDateTime) {
        RoleDO minister = roleService.getByCode(MINISTER_ROLE_CODE);
        List<Long> userIds = userRoleService.listUserIdByRoleId(minister.getId());

        userIds.forEach(userId -> {
            UserDO ministerUser = userService.getById(userId);
            List<UserDO> deptUsers = userService.getByDeptId(DisEnableStatusEnum.ENABLE, ministerUser.getDeptId());

            List<CompletableFuture<Void>> tasks = new ArrayList<>();

            if (ministerUser.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
                tasks.add(generateAndSendTeamReport(ministerUser, startDateTime, endDateTime));
            }

            AgentDataVisualListReq agentDataVisualListReq = AgentDataVisualListReq.builder()
                    .monthDate(reportDate)
                    .build();

            deptUsers.forEach(deptUser -> tasks.add(processDeptUser(deptUser, agentDataVisualListReq, reportDate, ministerUser)));

            CompletableFuture<Void> allTasks = CompletableFuture.allOf(tasks.toArray(new CompletableFuture[0]));
            allTasks.join();
        });
    }

    /**
     * 构建表格
     *
     * @param ministerUser  部长
     * @param startDateTime 起始时间
     * @param endDateTime   结束时间
     * @return CompletableFuture<Void>
     */

    private CompletableFuture<Void> generateAndSendTeamReport(UserDO ministerUser, LocalDateTime
            startDateTime, LocalDateTime endDateTime) {
        return CompletableFuture.runAsync(() -> {
            List<String[]> rows = new ArrayList<>();
            rows.add(new String[]{"平台", "注册", "新增", "转化率"});
            rows.add(new String[]{"----", "----", "----", "----"});

            List<AccountResp> accounts = accountService.getAccountsByUserId(ministerUser.getId(), DisEnableStatusEnum.ENABLE)
                    .stream()
                    .filter(AccountResp::getIsTeam)
                    .toList();

            int[] totals = {0, 0};
            TeamInfoReq teamInfoReq = TeamInfoReq.builder().startDate(startDateTime).endDate(endDateTime).build();

            List<CompletableFuture<Void>> futures = accounts.stream()
                    .map(accountResp -> teamService.getLatestTeamInfoAsync(accountResp, teamInfoReq)
                            .thenAcceptAsync(team -> {
                                int totalNewMember = team.getList().stream().mapToInt(TeamAccount::getSubMemberNum).sum();
                                int totalNewFirstDeposit = team.getList().stream().mapToInt(TeamAccount::getFirstDepositNum).sum();
                                synchronized (totals) {
                                    totals[0] += totalNewMember;
                                    totals[1] += totalNewFirstDeposit;
                                }
                                String percent = getPercent(totalNewFirstDeposit, totalNewMember);
                                synchronized (rows) {
                                    rows.add(new String[]{accountResp.getPlatformName(), String.valueOf(totalNewMember), String.valueOf(totalNewFirstDeposit), percent});
                                }
                            }, asyncTaskExecutor)
                    )
                    .toList();

            CompletableFuture<Void> allFutures = CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]));
            allFutures.join();

            rows.add(new String[]{"总注册", String.valueOf(totals[0]), String.valueOf(totals[1]), getPercent(totals[1], totals[0])});
            String message = TableFormatter.formatTableAsHtml(rows);
            telegramMessageService.sendMessage(ministerUser.getBotToken(), ministerUser.getReportIds(), message);
        }, asyncTaskExecutor).exceptionally(ex -> {
            log.error("Error generating and sending team report", ex);
            return null;
        });
    }

    /**
     * 查询部门所有用户的报表
     *
     * @param deptUser               部门下所有用户对象
     * @param agentDataVisualListReq 请求参数
     * @param reportDate             日期
     * @param ministerUser           上级
     * @return CompletableFuture<Void>
     */
    private CompletableFuture<Void> processDeptUser(UserDO deptUser, AgentDataVisualListReq
            agentDataVisualListReq, LocalDate reportDate, UserDO ministerUser) {
        return CompletableFuture.runAsync(() -> {
            List<AccountResp> currUserAccounts = accountService.getAccountsByUserId(deptUser.getId(), DisEnableStatusEnum.ENABLE)
                    .stream()
                    .filter(accountResp -> !accountResp.getIsTeam())
                    .toList();

            List<CompletableFuture<Statics>> futures = currUserAccounts.stream()
                    .map(currAccount -> agentDataVisualListService.getAgentDataVisualList(currAccount, agentDataVisualListReq)
                            .thenApplyAsync(agentData -> agentData.getCurData()
                                    .stream()
                                    .filter(data -> data.getStaticsDate().equals(reportDate))
                                    .findFirst()
                                    .orElseThrow(() -> new BusinessException("No data found for report date"))
                            )
                            .exceptionally(ex -> {
                                log.error("Error fetching data for account {}: {}", currAccount.getId(), ex.getMessage());
                                return null;
                            })
                    )
                    .toList();

            CompletableFuture<Void> userStaticsFuture = CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]));
            userStaticsFuture.thenRunAsync(() -> {
                List<Statics> agentDataList = futures.stream()
                        .map(future -> {
                            try {
                                return future.join();
                            } catch (Exception ex) {
                                log.error("Error joining future", ex);
                                return null;
                            }
                        })
                        .filter(Objects::nonNull)
                        .collect(Collectors.toList());

                // 构造消息体
                String message = telegramMessageService.buildDailyReportMessage(agentDataList);

                if (StrUtil.isNotBlank(message) && deptUser.getNeedNotify() == DisEnableStatusEnum.ENABLE) {
                    String botToken = StrUtil.isEmpty(deptUser.getBotToken()) ? ministerUser.getBotToken() : deptUser.getBotToken();
                    telegramMessageService.sendMessage(botToken, deptUser.getReportIds(), message);
                }
            }, asyncTaskExecutor).exceptionally(ex -> {
                log.error("Error collecting and processing data", ex);
                return null;
            }).join();
        }, asyncTaskExecutor).exceptionally(ex -> {
            log.error("Error processing dept user", ex);
            return null;
        });
    }


    private void saveData(LocalDate reportDate) {
        // 获取传入年月
        YearMonth inputYearMonth = YearMonth.from(reportDate);

        // 获取当前年月和上个月的年月
        YearMonth currentYearMonth = YearMonth.now();
        YearMonth previousYearMonth = currentYearMonth.minusMonths(1);

        if (inputYearMonth.equals(currentYearMonth) || inputYearMonth.equals(previousYearMonth)) {
            RoleDO minister = roleService.getByCode(MINISTER_ROLE_CODE);
            List<Long> userIds = userRoleService.listUserIdByRoleId(minister.getId());

            List<CompletableFuture<Void>> tasks = new ArrayList<>();

            userIds.forEach(userId -> {
                UserDO ministerUser = userService.getById(userId);
                List<UserDO> deptUsers = userService.getByDeptId(DisEnableStatusEnum.ENABLE, ministerUser.getDeptId());
                TeamFinanceReq teamFinanceReq = TeamFinanceReq.builder()
                        .pageNum(1)
                        .pageSize(999)
                        .commissionDate(reportDate)
                        .build();
                List<AccountResp> ministerUserAccounts = accountService.getAccountsByUserId(ministerUser.getId(), DisEnableStatusEnum.ENABLE)
                        .stream()
                        .filter(AccountResp::getIsTeam)
                        .toList();

                // 异步处理 ministerUserAccounts
                CompletableFuture<Void> ministerAccountsFuture = CompletableFuture.runAsync(() ->
                                ministerUserAccounts.parallelStream().forEach(accountResp -> {
                                    TeamFinancePagination<List<TeamAccountFinance>> financePagination = completableFutureFinanceService.getTeamFinance(accountResp, teamFinanceReq).join();
                                    List<FinanceDO> financeReqList = financePagination.getList().stream().map(finance -> {
                                        FinanceDO financeDO = new FinanceDO();
                                        BeanUtil.copyProperties(finance, financeDO);
                                        return financeDO;
                                    }).toList();
                                    financeService.addAll(financeReqList);
                                    FinanceSumReq financeSumReq = new FinanceSumReq();
                                    BeanUtil.copyProperties(financePagination.getTotalSumVo(), financeSumReq);
                                    financeSumService.add(financeSumReq);
                                }), asyncTaskExecutor)
                        .exceptionally(ex -> {
                            log.error("Error processing minister accounts", ex);
                            return null;
                        });
                tasks.add(ministerAccountsFuture);

                // 异步处理 deptUsers
                AgentDataVisualListReq agentDataVisualListReq = AgentDataVisualListReq.builder()
                        .monthDate(reportDate)
                        .build();

                deptUsers.forEach(deptUser -> {
                    CompletableFuture<Void> deptUserFuture = CompletableFuture.runAsync(() -> {
                        List<AccountResp> currUserAccounts = accountService.getAccountsByUserId(deptUser.getId(), DisEnableStatusEnum.ENABLE)
                                .stream()
                                .filter(accountResp -> !accountResp.getIsTeam())
                                .toList();

                        List<CompletableFuture<StatsDO>> futures = currUserAccounts.stream()
                                .map(currAccount -> agentDataVisualListService.getAgentDataVisualList(currAccount, agentDataVisualListReq)
                                        .thenApplyAsync(agentData -> {
                                            Statics statics = agentData.getCurData()
                                                    .stream()
                                                    .filter(data -> data.getStaticsDate().equals(reportDate))
                                                    .findFirst()
                                                    .orElseThrow(() -> new BusinessException("No data found for report date"));
                                            StatsDO statsDO = new StatsDO();
                                            BeanUtil.copyProperties(statics, statsDO);
                                            return statsDO;
                                        }, asyncTaskExecutor)
                                        .exceptionally(ex -> {
                                            log.error("Error fetching data for account {}: {}", currAccount.getId(), ex.getMessage());
                                            return null;
                                        })
                                ).toList();

                        List<StatsDO> list = futures.stream()
                                .map(CompletableFuture::join)
                                .filter(Objects::nonNull)
                                .collect(Collectors.toList());

                        statsService.addAll(list);
                    }, asyncTaskExecutor).exceptionally(ex -> {
                        log.error("Error processing dept user accounts", ex);
                        return null;
                    });
                    tasks.add(deptUserFuture);
                });
            });

            CompletableFuture<Void> allTasks = CompletableFuture.allOf(tasks.toArray(new CompletableFuture[0]));
            allTasks.join();
        } else {
            throw new BusinessException("只允许查询当月以及上个月的数据");
        }
    }


    public static String getPercent(int x, int y) {
        double d1 = x * 1.0;
        double d2 = y * 1.0;
        NumberFormat percentInstance = NumberFormat.getPercentInstance();
        percentInstance.setMinimumFractionDigits(2);
        return percentInstance.format(d1 / d2);
    }
}
