package com.zayac.admin.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.constant.ApiPathConstants;
import com.zayac.admin.req.MemberDetailsReq;
import com.zayac.admin.req.PayRecordsListReq;
import com.zayac.admin.req.team.TeamMemberListReq;
import com.zayac.admin.resp.Member;
import com.zayac.admin.resp.MemberPagination;
import com.zayac.admin.resp.Pagination;
import com.zayac.admin.resp.PayRecord;
import com.zayac.admin.resp.team.Team;
import com.zayac.admin.resp.team.TeamAccount;
import com.zayac.admin.resp.team.TeamAccountWithChange;
import com.zayac.admin.system.model.entity.UserDO;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.AccountService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Service;
import top.continew.starter.core.exception.BusinessException;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.ArrayList;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class DepositService {
    private final CompletableFutureWebClientService completableFutureWebClientService;
    private final TelegramMessageService telegramMessageService;
    private final AccountService accountService;

    public CompletableFuture<Void> processDeposits(UserDO minister,
                                                   AccountResp account,
                                                   Team currentTeam,
                                                   Team previousTeam,
                                                   LocalDate nowDate,
                                                   LocalDateTime nowDateTime,
                                                   Executor asyncTaskExecutor) {
        if (previousTeam == null || currentTeam.getFirstDepositNum() <= previousTeam.getFirstDepositNum()) {
            return CompletableFuture.completedFuture(null);
        }

        return processDepositRecords(minister, account, currentTeam, previousTeam, nowDate, nowDateTime, asyncTaskExecutor);
    }

    private CompletableFuture<Void> processDepositRecords(UserDO minister,
                                                          AccountResp account,
                                                          Team currentTeam,
                                                          Team previousTeam,
                                                          LocalDate nowDate,
                                                          LocalDateTime nowDateTime,
                                                          Executor asyncTaskExecutor) {
        PayRecordsListReq req = PayRecordsListReq.builder()
                .startDate(nowDate)
                .endDate(nowDate)
                .pageSize(100)
                .payState(2)
                .build();
        CompletableFuture<Pagination<List<PayRecord>>> paginationCompletableFuture = completableFutureWebClientService
                .fetchDataForAccount(account, ApiPathConstants.PAY_RECORDS_LIST_URL, req, new ParameterizedTypeReference<>() {
                });
        List<TeamAccountWithChange> changedTeamAccounts = findChangedTeamAccount(previousTeam, currentTeam);
        Set<String> changedAgentNames = changedTeamAccounts.stream()
                .filter(teamAccountWithChange -> teamAccountWithChange.getNewDepositNum() > 0)
                .map(TeamAccountWithChange::getAgentName)
                .collect(Collectors.toSet());
        return paginationCompletableFuture.thenApplyAsync(Pagination::getList, asyncTaskExecutor)
                .thenComposeAsync(payRecords ->
                        processPayRecords(payRecords, changedTeamAccounts, changedAgentNames, minister, account, nowDateTime, asyncTaskExecutor), asyncTaskExecutor)
                .exceptionally(ex -> {
                    log.error("Error processing deposits for account {}: {}", account.getId(), ex.getMessage());
                    return null;
                });
    }

    private CompletableFuture<Void> processPayRecords(List<PayRecord> payRecords,
                                                      List<TeamAccountWithChange> changedTeamAccounts,
                                                      Set<String> changedAgentNames,
                                                      UserDO minister,
                                                      AccountResp account,
                                                      LocalDateTime nowDateTime,
                                                      Executor asyncTaskExecutor) {
        Map<String, List<String>> agentNameWithNames = payRecords.stream()
                .filter(record -> record.getCreatedAt().isAfter(nowDateTime.minusHours(1)) && changedAgentNames
                        .contains(record.getAgentName()))
                .collect(Collectors.groupingBy(PayRecord::getAgentName, Collectors
                        .mapping(PayRecord::getName, Collectors.collectingAndThen(Collectors
                                .toSet(), ArrayList::new))));

        List<CompletableFuture<Void>> futures = agentNameWithNames.entrySet().stream()
                .map(entry -> processAgentRecords(entry.getKey(), entry.getValue(), changedTeamAccounts, payRecords, minister, account, asyncTaskExecutor))
                .toList();

        return CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]));
    }

    private CompletableFuture<Void> processAgentRecords(String agentName,
                                                        List<String> names,
                                                        List<TeamAccountWithChange> changedTeamAccounts,
                                                        List<PayRecord> payRecords,
                                                        UserDO minister,
                                                        AccountResp account,
                                                        Executor asyncTaskExecutor) {
        StringBuilder depositResults = new StringBuilder();
        AtomicInteger depositCounter = new AtomicInteger(0);

        TeamAccountWithChange targetTeamAccount = changedTeamAccounts.stream()
                .filter(teamAccount -> StrUtil.equals(teamAccount.getAgentName(), agentName))
                .findFirst()
                .orElseThrow(() -> new BusinessException(String.format("can not find agent name %s", agentName)));

        List<CompletableFuture<Void>> fetchFutures = names.stream()
                .map(name -> fetchMemberDetails(account, name, LocalDate.now(), asyncTaskExecutor).thenAcceptAsync(member ->
                        payRecords.stream()
                                .filter(record -> record.getCreatedAt().equals(member.getFirstPayAt()))
                                .findFirst()
                                .ifPresent(record -> {
                                    if (depositCounter.getAndIncrement() < targetTeamAccount.getNewDepositNum()) {
                                        depositResults.append(telegramMessageService
                                                .buildDepositResultsMessage(member.getName(), record.getScoreAmount()));
                                    }
                                }), asyncTaskExecutor).exceptionally(ex -> {
                    log.error("Error fetching details for member {}: {}", name, ex.getMessage());
                    return null;
                }))
                .toList();

        CompletableFuture<Void> allFetches = CompletableFuture.allOf(fetchFutures.toArray(new CompletableFuture[0]));

        return allFetches.thenRunAsync(() -> {
            if (!depositResults.isEmpty()) {
                String notification = telegramMessageService.buildDepositMessage(agentName, targetTeamAccount.getNewDepositNum(),
                        depositResults.toString(), targetTeamAccount.getFirstDepositNum());
                UserDO currUser = accountService.getUserByAccountUsername(agentName);
                if (currUser != null && DisEnableStatusEnum.ENABLE.equals(currUser.getNeedNotify())) {
                    String botToken = StrUtil.isEmpty(currUser.getBotToken()) ? minister.getBotToken() : currUser.getBotToken();
                    telegramMessageService.sendMessage(botToken, currUser.getRegAndDepIds(), notification);
                }
                telegramMessageService.sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, notification);
            }
        }, asyncTaskExecutor).exceptionally(ex -> {
            log.error("Error sending notification for account {}: {}", account.getUsername(), ex.getMessage());
            return null;
        });
    }

    private CompletableFuture<Member> fetchMemberDetails(AccountResp account, String name, LocalDate nowDate, Executor asyncTaskExecutor) {
        TeamMemberListReq memberListReq = TeamMemberListReq.builder()
                .name(name)
                .startDate(nowDate)
                .endDate(nowDate)
                .status(1)
                .build();

        CompletableFuture<MemberPagination<List<Member>>> memberFuture = completableFutureWebClientService
                .fetchDataForAccount(account, ApiPathConstants.MEMBER_TEAM_LIST_URL, memberListReq, new ParameterizedTypeReference<>() {
                });

        return memberFuture.thenApplyAsync(MemberPagination::getList, asyncTaskExecutor)
                .thenApplyAsync(list -> list.stream().findFirst().orElseThrow(() -> new RuntimeException("没有找到匹配的成员信息")), asyncTaskExecutor)
                .thenComposeAsync(member -> fetchDetailedMemberInfo(account, member.getId(), nowDate), asyncTaskExecutor);
    }

    private CompletableFuture<Member> fetchDetailedMemberInfo(AccountResp account, Long memberId, LocalDate nowDate) {
        MemberDetailsReq detailsReq = MemberDetailsReq.builder()
                .id(memberId)
                .startDate(nowDate)
                .endDate(nowDate)
                .build();

        return completableFutureWebClientService
                .fetchDataForAccount(account, ApiPathConstants.MEMBER_DETAIL_URL, detailsReq, new ParameterizedTypeReference<>() {
                });
    }

    private List<TeamAccountWithChange> findChangedTeamAccount(Team prevTeam, Team currTeam) {
        Map<Long, TeamAccount> team2AccountMap = currTeam.getList()
                .stream()
                .collect(Collectors.toMap(TeamAccount::getId, account -> account));

        return prevTeam.getList()
                .stream()
                .filter(account1 -> team2AccountMap.containsKey(account1.getId()))
                .map(account1 -> {
                    TeamAccount account2 = team2AccountMap.get(account1.getId());
                    TeamAccountWithChange changedAccount = new TeamAccountWithChange();
                    BeanUtil.copyProperties(account2, changedAccount);
                    if (account1.getFirstDepositNum() != account2.getFirstDepositNum()) {
                        changedAccount.setNewDepositNum(account2.getFirstDepositNum() - account1.getFirstDepositNum());
                    }
                    if (account1.getSubMemberNum() != account2.getSubMemberNum()) {
                        changedAccount.setNewRegisterNum(account2.getSubMemberNum() - account1.getSubMemberNum());
                    }
                    return changedAccount;
                })
                .collect(Collectors.toList());
    }
}
